#!/bin/sh


# Stop all labels: The installer reads a list of disks and partitions from
# the kernel that uses spaces as delimiter characters. If a label contains
# space characters, the layout of this list is mangled and the installer
# can't proceed.
#
# However, geom_label can't be disabled completely since FreeSBIE uses it
# to identify the disc it booted from.
#
find /dev/ufs /dev/msdosfs /dev/iso9660 /dev/ext2fs /dev/reiserfs /dev/ntfs /dev/label  ! -type 'd'  -exec glabel stop {} ';' >/dev/null 2>&1


# Create additional service users missing in live systems by default
for name in "avahi:558" "messagebus:556" "polkit:562" "haldaemon:560" "cups:193" ; do
	user="${name%:*}"
	uid="${name#*:}"
	group="${user}"
	gid="${uid}"

	# Check if group exists and create if necessary
	if ! /usr/sbin/pw group show "${group}" >/dev/null 2>&1 ; then
		/usr/sbin/pw group add "${group}" -g "${gid}"
	fi

	# Check if user exists and create if necessary
	if ! /usr/sbin/pw user show "${user}" >/dev/null 2>&1 ; then
		/usr/sbin/pw user add "${user}" -u "${uid}" -g "${group}" -h - -d "/nonexistent" -s "/sbin/nologin"
	fi
done

# Create DBUS UUID
mkdir -p /var/db/dbus/
/usr/local/bin/dbus-uuidgen --ensure


# Exit immediately when a graphical environment
# has already been configured
[ -f .xorg-confaccepted ] && exit


# The actual /media directory is mounted read-only, create a subdirectory
# in /root and mount it using nullfs.
mkdir -m 0755 ~/.media && mount_nullfs ~/.media /media

# Copy KDE configuration from skeleton
rm -fr ~/.kde/share/config/
cp -pR /usr/share/skel/dot.kde/share ~/.kde
rm -fr ~/.qt/
cp -pR /usr/share/skel/dot.qt ~/.qt
cp -p /usr/share/skel/dot.gtkrc ~/.gtkrc
cp -p /usr/share/skel/dot.gtkrc-2.0 ~/.gtkrc-2.0
cp -pR /usr/share/skel/dot.mozilla ~/.mozilla

# Disable arts: Reduces loading time and prevents
# an error message when no sound card is present
mkdir -p ~/.kde/share/config/
printf '[Arts]\nStartServer=false' > ~/.kde/share/config/kcmartsrc

# Disable desktop icons: Weird bug that leads to a KDesktop/KLauncher
# error message in some cases
printf '\n[General]\nEnabled=false' >> ~/.kde/share/config/kdesktoprc

if [ ! -x /usr/local/bin/X ]; then
	echo 'Graphics Error: X11 not available!'
	exit
fi

XCFG=/etc/X11/xorg.conf

# Created later by "X -configure"
XCFG_AUTO=/root/xorg.conf.new

# Default configuration without 'Device' section
XCFG_FALLBACK=/etc/X11/xorg.conf.fallback

# Last resort configuration for VMs and hopeless cases
XCFG_LASTRESORT=/etc/X11/xorg.conf.lastresort

# Log file for output of startx
STARTX_LOG=/var/log/Xsession

# Device section for xorg.conf (use with printf)
DEVSECTION='\nSection "Device"\n\tIdentifier\t"Card0"\n\tDriver\t\t"%s"\nEndSection\n'

dialog --msgbox \
'In the next step, the graphical environment will be initialized.

If your monitor does not display a proper image or shows "signal out
of range" or a similar error, please press the CTRL, ALT and BACKSPACE
keys together to try an alternative configuration.

Press Enter to continue.' 11 74

startxorg()
{
	startx >"${STARTX_LOG}" 2>&1

	# .xorg-confaccepted is created when the user is
	# satisfied with the detected graphics settings
	[ -f .xorg-confaccepted ] && exit
}


while true; do
	# X -configure only displays a black screen and hangs when using
	# QEMU or VMware. Use the fallback configuration for those. There
	# were reports that Virtual PC is also affected, but it seems to
	# work fine with Virtual PC 2004 + SP1 under Windows XP.
	if ! (pciconf -lv | fgrep -qi 'vmware' || \
			sysctl -n dev.acd.0.%desc | fgrep -qi 'qemu'); then

		# Have Xorg try to find a suitable configuration
		unset XCFG_AUTO_SUCCESS
		if /usr/local/bin/X -configure >/dev/null 2>&1 ; then
			XCFG_AUTO_SUCCESS=true

			echo '[*] X auto configuration appears to be successful.' >&2
			mv "${XCFG_AUTO}" "${XCFG}"

			# Slip 'AllowEmptyInput' option into ServerLayout section
			sed -i '' 's|^Section "ServerLayout".*$|Section "ServerLayout"\
			Option	"AllowEmptyInput"	"off"|g' "${XCFG}"

			# Load "nvidia" kernel module for supported cards
			if egrep -q 'Driver +"nvidia"' "${XCFG}"; then
				kldstat -qm 'nvidia' || kldload 'nvidia'
			fi

			mdet=$(sed -n '/^Section "Monitor"/,/^EndSection/p' "${XCFG}" | awk  '$1 == "VendorName" { print substr($2, 2, 7) }')
			if [ "$mdet" = 'Monitor' ]; then
				dialog --yesno \
'The specifications for your monitor could not be probed.

Do you want to try to find the correct settings automatically?
If you have a very old monitor, it might be damaged.' 8 66

				[ "$?" = 0 ] && startxorg
			else
				startxorg
			fi
		else
			echo '[*] X auto configuration failed.' >&2
		fi


		# Auto configuration succeeded, but either the monitor could not be
		# detected using DDC and the user doesn't want Xorg to try
		# resolutions or something else went wrong. Try default monitor settings.
		if [ "${XCFG_AUTO_SUCCESS}" ]; then
			echo '[*] Trying different monitor settings...' >&2

			sed -i '' '/^Section "Monitor"/,/^EndSection/d' "${XCFG}"
			sed -n '/^Section "Monitor"/,/^EndSection/p' "${XCFG_FALLBACK}" >> "${XCFG}"

			startxorg
		fi


		echo '[*] Trying conservative configuration.' >&2
		cp "${XCFG_FALLBACK}" "${XCFG}"
		printf "${DEVSECTION}" 'vesa' >> "${XCFG}"
		startxorg
	fi

	echo '[*] Trying very conservative configuration.' >&2
	cp "${XCFG_LASTRESORT}" "${XCFG}"
	printf "${DEVSECTION}" 'vesa' >> "${XCFG}"
	startxorg

	if ! dialog --yesno \
'All possible configurations have been tried now.

Do you want to start with the first one again?
Choose "No" to drop to a command-line interface.' 8 52 ; then
		clear
		printf 'Use \033[1;37;40mhalt\033[0m or \033[1;37;40mreboot\033[0m to shut down or restart your computer.\n\n'

		exit
	fi
done

